package main

import (
	"crypto/md5"
	"encoding/hex"
	"errors"
	"os"
	"sync"
	"time"

	"pkg.deepin.io/lib/dbus"
	"pkg.deepin.io/lib/log"
)

var logger = log.NewLogger("deepin/cloudprint/auth/agent")

var (
	dbusDest = "com.deepin.cloudprint.Auth.Agent"
	dbusPath = "/com/deepin/cloudprint/Auth/Agent"
	dbusIfc  = "com.deepin.cloudprint.Auth.Agent"
)

type printerAuth struct {
	mu       sync.Mutex
	authcode chan string
}

type cloudPrintAuth struct {
	mu              sync.Mutex
	auths           map[string]*printerAuth
	RequestAuthCode func(printerAddr string)
	JobFinish       func(printer, job, err string)
}

func (cpa *cloudPrintAuth) GetDBusInfo() dbus.DBusInfo {
	return dbus.DBusInfo{
		Dest:       dbusDest,
		ObjectPath: dbusPath,
		Interface:  dbusIfc,
	}
}

func (cpa *cloudPrintAuth) NotifyJobFinish(printerID, job, err string) error {
	return dbus.Emit(cpa, "JobFinish", printerID, job, err)
}

func (cpa *cloudPrintAuth) SetAuthCode(printerAddr, authcode string) error {
	logger.Info("SetAuthCode of", printerAddr)
	auth := cpa.auths[printerAddr]
	if nil == auth {
		return nil
	}

	auth.authcode <- authcode
	return nil
}

//GetAuthCode will block
func (cpa *cloudPrintAuth) GetAuthCode(printerAddr string) (string, error) {
	logger.Info("GetAuthCode of", printerAddr)
	cpa.mu.Lock()
	auth := cpa.auths[printerAddr]
	if nil == auth {
		cpa.auths[printerAddr] = &printerAuth{authcode: make(chan string, 1)}
		auth = cpa.auths[printerAddr]
	}
	cpa.mu.Unlock()

	auth.mu.Lock()
	defer auth.mu.Unlock()

	dbus.Emit(cpa, "RequestAuthCode", printerAddr)
	timer := time.NewTimer(time.Minute * 3)
	for {
		select {
		case code := <-auth.authcode:
			return code, nil
		case <-timer.C:
			return "", errors.New("Wait Authcode Time Out")
		}
	}
	return "", errors.New("Can Not Get AuthCode")
}

func signaturePrinterID(id string) string {
	sig := md5.New()
	sig.Write([]byte(id))
	return hex.EncodeToString(sig.Sum(nil))
}

func main() {
	auth := &cloudPrintAuth{
		auths: map[string]*printerAuth{},
	}
	err := dbus.InstallOnSystem(auth)
	if nil != err {
		logger.Error("InstallOnSystem Failed", auth.GetDBusInfo(), err)
	}

	dbus.DealWithUnhandledMessage()
	if err := dbus.Wait(); nil != err {
		logger.Error("dbus.Wait() Failed: ", err)
		os.Exit(1)
	}
	logger.Warning("dbus Exit")
	os.Exit(0)
}
