package main

import (
	"encoding/base64"
	"encoding/json"
	"io"
	"io/ioutil"
	"net/url"
	"os"
	"os/user"
	"path"
	"strings"
	"sync"

	"git.apache.org/thrift.git/lib/go/thrift"
	"pkg.deepin.io/cloudprint/rpc/cloudprint"
	"pkg.deepin.io/cloudprint/rpc/print"
)

var (
	configFilename = ".config/deepin/cloud-print/server.json"
)

var _jobControl = make(chan print.JobCommand, 1)

var (
	_lock           sync.Mutex
	_currentClient  *cloudprint.CloudPrintClient
	_currentAuth    *cloudprint.Auth
	_currentPrinter *print.Printer
	_currentJobID   print.JobID
)

type ServerConfig struct {
	Addr     string
	Authcode string
}

func readUserAuth(owner *user.User, serverURL string) (auth *cloudprint.Auth) {
	auth = &cloudprint.Auth{}
	cfg, err := os.Open(path.Join(owner.HomeDir, configFilename))
	if nil != err {
		logger.Println("Read User Config Failed:", owner.Username, err)
		return
	}
	cfgData, err := ioutil.ReadAll(cfg)
	if nil != err {
		logger.Println("Read User Config File Failed:", cfg.Name(), err)
		return
	}

	var cfgs map[string]*ServerConfig
	err = json.Unmarshal(cfgData, &cfgs)
	if nil != err {
		logger.Println("Parse User Config File Failed:", cfg.Name(), err)
		return
	}
	server := cfgs[serverURL]

	if nil != server {
		auth.Authcode = server.Authcode
	}
	return
}

func cloudPrintFile(owner *user.User, dcpURL string, title string, filePath string, op *print.Option) error {
	logger.Println(debugOption(op))
	u, err := url.Parse(dcpURL)
	if nil != err {
		logger.Println("Parse dcpURL failed:", err, dcpURL)
		return err
	}

	serverURL := u.Scheme + "://" + u.Host

	trans, err := thrift.NewTSocket(u.Host)
	if nil != err {
		logger.Println("Error resolving address:", err, u.Host)
		return err
	}
	defer trans.Close()

	protocolFactory := thrift.NewTBinaryProtocolFactoryDefault()
	client := cloudprint.NewCloudPrintClientFactory(trans, protocolFactory)
	if err := trans.Open(); err != nil {
		logger.Println("Open socket failed:", err, u.Host)
		return err
	}

	auth := readUserAuth(owner, serverURL)
	verfiy, err := client.Verfiy(auth)
	if nil != err {
		logger.Println("Verfiy user failed:", err, verfiy)
		return err
	}

	if !verfiy {
		logger.Println("Update user auth", owner.Username)
		code, err := getAuthCode(strings.Split(u.Host, ":")[0])
		if nil != err {
			logger.Println("RequireAuthCode user auth failed", owner.Username, err)
			return err
		}
		auth.Authcode = code
	}

	printerID := strings.Replace(u.Path, "/", "", -1)
	printer, err := client.GetPrinter(auth, print.PrinterID(printerID))
	if nil != err {
		logger.Println("Error GetPrinter:", printerID, err)
		return err
	}

	jobID, err := client.CreateJob(auth, printer, title, op)
	if nil != err {
		logger.Println("Error CreateJob:", err, jobID)
		return err
	}

	_lock.Lock()
	_currentClient = client
	_currentAuth = auth
	_currentPrinter = printer
	_currentJobID = jobID
	_lock.Unlock()

	logger.Println("Start Job:", jobID)
	buf := make([]byte, 1024*1024)
	f, err := os.Open(filePath)
	if nil != err {
		logger.Println("Open file failed:", filePath, err)
		return err
	}
	defer f.Close()
	for {
		n, err := f.Read(buf)
		logger.Println("Read bytes:", n, err)
		if err != nil && err != io.EOF {
			logger.Println("Error Read file:", err, filePath)
			return err
		}
		if n != 0 {
			logger.Println("WriteJob bytes:", n, err)
			client.WriteJob(auth, printer, jobID, int32(n), base64.StdEncoding.EncodeToString(buf[0:n]))
			logger.Println("WriteJob bytes Finish:", n, err)
		}
		if err == io.EOF {
			logger.Println("Read file end:", n, err)
			break
		}
	}

	logger.Println("Call  FinishJob")
	return client.FinishJob(auth, printer, jobID)
}

func cancelCurrentJob() {
	_lock.Lock()
	defer _lock.Unlock()
	if nil == _currentClient || nil == _currentAuth || nil == _currentPrinter {
		logger.Println("Job not create now")
		return
	}
	err := _currentClient.CancelJob(_currentAuth, _currentPrinter, _currentJobID)
	logger.Println("Call CancelJob", err)
}
