package main

import (
	"fmt"
	"io"
	"log"
	"os"
	"os/exec"
	"os/user"
	"path"
	"strconv"
)

var _logFile *os.File
var logger *log.Logger

var (
	userCacheDir = ".cache/deepin/cloud-print/PDF"
)

func init() {
	var err error
	_logFile, err = os.OpenFile("/var/log/cups/dcp_log", os.O_RDWR|os.O_APPEND|os.O_CREATE, 0777)
	if nil != err {
		fmt.Println("open log file failed", _logFile, err)
	}
	logger = log.New(_logFile, "[DeepinCloudPrint] ", log.Lshortfile)
}

// IsExist returns whether a file or directory exists.
func IsExist(path string) bool {
	_, err := os.Stat(path)
	return err == nil || os.IsExist(err)
}

func main() {
	var err error
	defer _logFile.Close()

	logger.Println("Start dcp with:", os.Args)

	registerSignalHandle()

	argc := len(os.Args)
	if 1 == argc {
		fmt.Println("file dcp:/ \"Deepin Cloud Printer\" \"DCP\" \"MFG:Generic;MDL:Deepin Cloud Printer;DES:Generic Deepin Cloud Printer;CLS:PRINTER;CMD:POSTSCRIPT;\"")
		logger.Println("Send identification string")
		os.Exit(0)
	}

	if argc < 6 || argc > 7 {
		fmt.Println("Usage: dcp job-id user title copies options [file]")
		logger.Println("Arguments number error")
		os.Exit(1)
	}

	dcpURL := os.Args[0]
	jobID := os.Args[1]
	userName := os.Args[2]
	title := os.Args[3]
	copies, _ := strconv.Atoi(os.Args[4])
	options := os.Args[5]

	logger.Println("caller: ", fmt.Sprint(user.Current()))
	owner, err := user.Lookup(userName)
	if nil != err {
		logger.Println("Find user failed: ", userName, err)
		os.Exit(1)
	}
	logger.Println("owner: ", owner, err)

	logger.Println("Start new job", jobID, title)

	inputFile := os.Stdin
	if 7 == argc {
		if inputFile, err = os.Open(os.Args[6]); nil != err {
			logger.Println("Open input file failed: ", inputFile, err)
			os.Exit(1)
		}
	}
	defer inputFile.Close()

	spoolDir := "/var/spool/dcp/SPOOL"
	os.MkdirAll(spoolDir, 0644)
	spoolFileName := fmt.Sprintf("/var/spool/dcp/SPOOL/dcp-%v", os.Getpid())

	destFile, err := os.OpenFile(spoolFileName, os.O_TRUNC|os.O_CREATE|os.O_RDWR, 0644)
	if nil != err {
		logger.Println("Create spool file failed: ", spoolFileName, err)
		os.Exit(1)
	}
	defer destFile.Close()
	defer os.Remove(spoolFileName)

	_, err = io.Copy(destFile, inputFile)
	if nil != err {
		logger.Println("Copy spool file failed: ", spoolFileName, err)
		os.Exit(1)
	}

	cacheDir := path.Join(owner.HomeDir, userCacheDir)
	if !IsExist(cacheDir) {
		os.MkdirAll(cacheDir, 0777)
	}

	jobName := fmt.Sprintf("Job_%v.pdf", jobID)
	outFileName := path.Join(cacheDir, jobName)
	defer os.Remove(outFileName)

	gsCmd := []string{"-q", "-dCompatibilityLevel=1.4",
		"-dNOPAUSE", "-dBATCH", "-dSAFER",
		"-sDEVICE=pdfwrite", "-sOutputFile=" + outFileName,
		"-dAutoRotatePages=/PageByPage", "-dAutoFilterColorImages=false",
		"-dColorImageFilter=/FlateEncode", "-dPDFSETTINGS=/prepress",
		"-c", ".setpdfwrite", "-f", spoolFileName}
	gsCall := exec.Command("/usr/bin/gs", gsCmd...)
	logger.Println("Run gs with: ", gsCall.Args)
	if err = gsCall.Start(); err != nil {
		logger.Println("Start gs failed", err)
		os.Exit(1)
	}

	if err = gsCall.Wait(); nil != err {
		logger.Println("Run gs failed", err)
		os.Exit(1)
	}

	option := parseCUPSOptions(options)
	option.Copyies = int32(copies)
	err = cloudPrintFile(owner, dcpURL, title, outFileName, &option)
	logger.Println("cloudPrintFile ret: ", err)
	if nil != err {
		notifyPrintJob(title, err.Error())
		os.Exit(1)
	}
	notifyPrintJob(title, "")

	logger.Println("End new job")
}
